import sys
from pathlib import Path
from test_exporter import check_output
from util import TEST_DIR, template_path_prefix, gen_option_output_map
from mnrs_gen import main as gen_main


class TestWrapper():
    def test_addrmap_passthrough(self, tmp_path: Path):
        sys.argv = [
            "", str(TEST_DIR/"rdl"/"test_addrmap.rdl"), "-o", str(tmp_path), "--addrmap", "--template", f"{template_path_prefix}/TestAddrMap.jinja"]
        gen_main()
        check_output(tmp_path, 'addrmap')

    def test_naked_regfile(self, tmp_path: Path):
        sys.argv = [
            "", str(TEST_DIR/"rdl"/"other_test_regfiles.rdl"), "-o", str(tmp_path), "--regfile", "--template", f"{template_path_prefix}/TestRegfile.jinja"]
        gen_main()
        with open(tmp_path/"gen"/"read_write_test_regs.h", encoding="utf8") as generated_file, \
                open(TEST_DIR/"references"/"read_write_test.h", encoding="utf8") as reference_file:
            gen_content = generated_file.read()
            ref_content = reference_file.read()
            assert gen_content == ref_content

    def test_multiple_naked_regfiles(self, tmp_path: Path):
        sys.argv = [
            "", str(TEST_DIR/"rdl"/"test_regfiles.rdl"), "-o", str(tmp_path), "--regfile", "--template", f"{template_path_prefix}/TestRegfile.jinja"]
        gen_main()
        for ref in gen_option_output_map["regfile"]:
            with open(tmp_path/ref, encoding="utf8") as generated_file, \
                    open(TEST_DIR/"references"/ref, encoding="utf8") as reference_file:
                gen_content = generated_file.read()
                ref_content = reference_file.read()
                assert gen_content == ref_content
